/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2015-2023                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


#ifndef CUBELIB_BSPOP_HYBRID_MPI_COMMUNICATION_EFFICIENCY_TEST_H
#define CUBELIB_BSPOP_HYBRID_MPI_COMMUNICATION_EFFICIENCY_TEST_H

#include <string>
#include "PerformanceTest.h"

namespace bscanalysis
{
class BSPOPHybridOMPCommunicationEfficiencyTest;
class BSPOPHybridMPICommunicationEfficiencyTest : public popcalculation::PerformanceTest
{
    friend class BSPOPHybridOMPCommunicationEfficiencyTest;
private:

    cube::Metric* max_runtime;
    cube::Metric* max_non_mpi_time;

    cube::list_of_metrics max_non_mpi_metrics;

    void
    calculate();

    void
    calculate( const cube::list_of_cnodes& );

protected:


    virtual
    const std::string&
    getCommentText() const;

    inline
    virtual
    std::string
    getHelpUrl() const
    {
        std::string active_text =
            "MPI Communication Efficiency can be evaluated directly by following formula:\n"
            "\tMPI Communication Efficiency = max(time outside of MPI)/max(runtime)\n"
            "This metric identifies when code is inefficient because it spends a large amount\n"
            "of time communicating rather than performing useful computations. ";
        std::string not_active_text =
            "MPI Communication Efficiency metric is a basic POP metric and is available for every Score-P/Scalasca measurement. \n"
            "If Cube Report was produced by another tool than Score-P/Scalasca, it might have missing metric Time.\n"
            "In this case POP analysis is not possible.";

        return active_text;
    }

    virtual
    void
    adjustForTest( cube::CubeProxy* cube ) const;


public:
    BSPOPHybridMPICommunicationEfficiencyTest(      cube::CubeProxy* );

    virtual
    ~BSPOPHybridMPICommunicationEfficiencyTest()
    {
    };
// ------ overview tests ---------

    void
    applyCnode( const cube::list_of_cnodes& cnodes,
                const bool                  direct_calculation = false  );


    bool
    isActive() const;

    bool
    isIssue() const;
};
};
#endif // ADVISER_RATING_WIDGET_H
