/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2015-2023                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


#ifndef CUBELIB_BSPOP_HYBRID_MPI_LOAD_BALANCE_TEST_H
#define CUBELIB_BSPOP_HYBRID_MPI_LOAD_BALANCE_TEST_H

#include <string>
#include "PerformanceTest.h"
#include "BSPOPHybridMPICommunicationEfficiencyTest.h"


namespace bscanalysis
{
class BSPOPHybridOMPLoadBalanceEfficiencyTest;
class BSPOPHybridMPILoadBalanceTest : public popcalculation::PerformanceTest
{
    friend class BSPOPHybridOMPLoadBalanceEfficiencyTest;
private:

    cube::Metric* non_mpi_time;
    cube::Metric* max_non_mpi_time;

    cube::list_of_metrics                      max_non_mpi_metrics;
    BSPOPHybridMPICommunicationEfficiencyTest* mpi_comm_eff;



protected:


    virtual
    const std::string&
    getCommentText() const;

    inline
    virtual
    std::string
    getHelpUrl() const
    {
        std::string active_text =
            "MPI Load Balance Efficiency can be computed as follows: \n"
            "\tMPI Load Balance Efficiency = avg(time outside of MPI)/ max(time outside of MPI)\n";
        std::string not_active_text =
            "MPI Load Balance Efficiency metric is a basic POP metric and is available for every Score-P/Scalasca measurement. \n"
            "If Cube Report was produced by another tool than Score-P/Scalasca, it might have missing metric Time.\n"
            "In this case POP analysis is not possible.";

        return active_text;
    }

    virtual
    void
    adjustForTest( cube::CubeProxy* cube ) const;

public:
    BSPOPHybridMPILoadBalanceTest( cube::CubeProxy*,
                                   BSPOPHybridMPICommunicationEfficiencyTest* );

    virtual
    ~BSPOPHybridMPILoadBalanceTest()
    {
    };

    virtual
    double
    analyze( const cube::list_of_cnodes& cnodes,
             cube::LocationGroup*        _lg = nullptr ) const;




// ------ overview tests ---------

    void
    applyCnode( const cube::list_of_cnodes& cnodes,
                const bool                  direct_calculation = false );



    bool
    isActive() const;

    bool
    isIssue() const;

    virtual
    std::list<PerformanceTest*>
    getPrereqs()
    {
        std::list<PerformanceTest*> prereqs;
        if ( mpi_comm_eff != nullptr )
        {
            prereqs.push_back( mpi_comm_eff );
        }
        return prereqs;
    }
};
};
#endif // BSPOPHybrid_IMBALANCE_TEST_H
